@@ -26,59 +26,31 @@
 import sys
 import platform
 
-VERSION = (1, 3, 4)
+VERSION = (1, 4, 0)
 
 try:
     from rhodecode.lib import get_current_revision
-    _rev = get_current_revision()
-    if _rev:
+    _rev = get_current_revision(quiet=True)
+    if _rev and len(VERSION) > 3:
         VERSION += ('dev%s' % _rev[0],)
 except ImportError:
     pass
 
 __version__ = ('.'.join((str(each) for each in VERSION[:3])) +
                '.'.join(VERSION[3:]))
-__dbversion__ = 5  # defines current db version for migrations
+__dbversion__ = 6  # defines current db version for migrations
 __platform__ = platform.system()
 __license__ = 'GPLv3'
 __py_version__ = sys.version_info
+__author__ = 'Marcin Kuzminski'
+__url__ = 'http://rhodecode.org'
 
 PLATFORM_WIN = ('Windows')
-PLATFORM_OTHERS = ('Linux', 'Darwin', 'FreeBSD', 'OpenBSD', 'SunOS')
+PLATFORM_OTHERS = ('Linux', 'Darwin', 'FreeBSD', 'OpenBSD', 'SunOS') #depracated
 
-requirements = [
-    "Pylons==1.0.0",
-    "Beaker==1.6.3",
-    "WebHelpers==1.3",
-    "formencode==1.2.4",
-    "SQLAlchemy==0.7.6",
-    "Mako==0.6.2",
-    "pygments>=1.4",
-    "whoosh>=2.3.0,<2.4",
-    "celery>=2.2.5,<2.3",
-    "babel",
-    "python-dateutil>=1.5.0,<2.0.0",
-    "dulwich>=0.8.4,<0.9.0",
-    "webob==1.0.8",
-    "markdown==2.1.1",
-    "docutils==0.8.1",
-]
+is_windows = __platform__ in PLATFORM_WIN
+is_unix = not is_windows
 
-if __py_version__ < (2, 6):
-    requirements.append("simplejson")
-    requirements.append("pysqlite")
-
-if __platform__ in PLATFORM_WIN:
-    requirements.append("mercurial>=2.1,<2.2")
-else:
-    requirements.append("py-bcrypt")
-    requirements.append("mercurial>=2.1,<2.2")
-
-
-def get_version():
-    """Returns shorter version (digit parts only) as string."""
-
-    return '.'.join((str(each) for each in VERSION[:3]))
 
 BACKENDS = {
     'hg': 'Mercurial repository',
@@ -86,6 +58,7 @@ BACKENDS = {
 }
 
 CELERY_ON = False
+CELERY_EAGER = False
 
 # link to config for pylons
 CONFIG = {}
